@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Python Markdown
 
@@ -25,7 +24,6 @@ Python-Markdown Regression Tests
 Tests of the various APIs with the python markdown lib.
 """
 
-from __future__ import unicode_literals
 import unittest
 import sys
 import os
@@ -36,17 +34,10 @@ from logging import DEBUG, WARNING, CRITICAL
 import yaml
 import tempfile
 from io import BytesIO
+import xml.etree.ElementTree as etree
 from xml.etree.ElementTree import ProcessingInstruction
 
 
-PY3 = sys.version_info[0] == 3
-
-
-if not PY3:
-    def bytes(string, encoding):
-        return string.encode(encoding)
-
-
 class TestMarkdownBasics(unittest.TestCase):
     """ Tests basics of the Markdown class. """
 
@@ -132,7 +123,7 @@ class TestBlockParser(unittest.TestCase):
 
     def testParseChunk(self):
         """ Test BlockParser.parseChunk. """
-        root = markdown.util.etree.Element("div")
+        root = etree.Element("div")
         text = 'foo'
         self.parser.parseChunk(root, text)
         self.assertEqual(
@@ -144,8 +135,8 @@ class TestBlockParser(unittest.TestCase):
         """ Test BlockParser.parseDocument. """
         lines = ['#foo', '', 'bar', '', '    baz']
         tree = self.parser.parseDocument(lines)
-        self.assertTrue(isinstance(tree, markdown.util.etree.ElementTree))
-        self.assertTrue(markdown.util.etree.iselement(tree.getroot()))
+        self.assertIsInstance(tree, etree.ElementTree)
+        self.assertIs(etree.iselement(tree.getroot()), True)
         self.assertEqual(
             markdown.serializers.to_xhtml_string(tree.getroot()),
             "<div><h1>foo</h1><p>bar</p><pre><code>baz\n</code></pre></div>"
@@ -220,7 +211,7 @@ class TestHtmlStash(unittest.TestCase):
         self.assertEqual(self.stash.rawHtmlBlocks, [])
 
 
-class Item(object):
+class Item:
     """ A dummy Registry item object for testing. """
     def __init__(self, data):
         self.data = data
@@ -239,7 +230,7 @@ class RegistryTests(unittest.TestCase):
         r = markdown.util.Registry()
         r.register(Item('a'), 'a', 20)
         self.assertEqual(len(r), 1)
-        self.assertTrue(isinstance(r, markdown.util.Registry))
+        self.assertIsInstance(r, markdown.util.Registry)
 
     def testRegisterWithoutPriority(self):
         r = markdown.util.Registry()
@@ -256,20 +247,20 @@ class RegistryTests(unittest.TestCase):
 
     def testIsSorted(self):
         r = markdown.util.Registry()
-        self.assertFalse(r._is_sorted)
+        self.assertIs(r._is_sorted, False)
         r.register(Item('a'), 'a', 20)
         list(r)
-        self.assertTrue(r._is_sorted)
+        self.assertIs(r._is_sorted, True)
         r.register(Item('b'), 'b', 21)
-        self.assertFalse(r._is_sorted)
+        self.assertIs(r._is_sorted, False)
         r['a']
-        self.assertTrue(r._is_sorted)
+        self.assertIs(r._is_sorted, True)
         r._is_sorted = False
         r.get_index_for_name('a')
-        self.assertTrue(r._is_sorted)
+        self.assertIs(r._is_sorted, True)
         r._is_sorted = False
         repr(r)
-        self.assertTrue(r._is_sorted)
+        self.assertIs(r._is_sorted, True)
 
     def testDeregister(self):
         r = markdown.util.Registry()
@@ -281,11 +272,11 @@ class RegistryTests(unittest.TestCase):
         self.assertEqual(len(r), 2)
         r.deregister('c', strict=False)
         self.assertEqual(len(r), 1)
-        # deregister non-existant item with strict=False
+        # deregister non-existent item with strict=False
         r.deregister('d', strict=False)
         self.assertEqual(len(r), 1)
         with self.assertRaises(ValueError):
-            # deregister non-existant item with strict=True
+            # deregister non-existent item with strict=True
             r.deregister('e')
         self.assertEqual(list(r), ['a'])
 
@@ -293,9 +284,9 @@ class RegistryTests(unittest.TestCase):
         r = markdown.util.Registry()
         item = Item('a')
         r.register(item, 'a', 20)
-        self.assertTrue('a' in r)
-        self.assertTrue(item in r)
-        self.assertFalse('b' in r)
+        self.assertIs('a' in r, True)
+        self.assertIn(item, r)
+        self.assertNotIn('b', r)
 
     def testRegistryIter(self):
         r = markdown.util.Registry()
@@ -325,50 +316,16 @@ class RegistryTests(unittest.TestCase):
         r = markdown.util.Registry()
         with self.assertRaises(TypeError):
             r[0] = 'a'
-        # TODO: restore this when deprecated __setitem__ is removed.
-        # with self.assertRaises(TypeError):
-        #     r['a'] = 'a'
-        # TODO: remove this when deprecated __setitem__ is removed.
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-
-            r['a'] = Item('a')
-            self.assertEqual(list(r), ['a'])
-            r['b'] = Item('b')
-            self.assertEqual(list(r), ['a', 'b'])
-            r['a'] = Item('a1')
-            self.assertEqual(list(r), ['a1', 'b'])
-
-            # Check the warnings
-            self.assertEqual(len(w), 3)
-            self.assertTrue(all(issubclass(x.category, DeprecationWarning) for x in w))
+        with self.assertRaises(TypeError):
+            r['a'] = 'a'
 
     def testRegistryDelItem(self):
         r = markdown.util.Registry()
         r.register(Item('a'), 'a', 20)
         with self.assertRaises(TypeError):
             del r[0]
-        # TODO: restore this when deprecated __del__ is removed.
-        # with self.assertRaises(TypeError):
-        #     del r['a']
-        # TODO: remove this when deprecated __del__ is removed.
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-
-            r.register(Item('b'), 'b', 15)
-            r.register(Item('c'), 'c', 10)
-            del r['b']
-            self.assertEqual(list(r), ['a', 'c'])
+        with self.assertRaises(TypeError):
             del r['a']
-            self.assertEqual(list(r), ['c'])
-            with self.assertRaises(TypeError):
-                del r['badname']
-            del r['c']
-            self.assertEqual(list(r), [])
-
-            # Check the warnings
-            self.assertEqual(len(w), 3)
-            self.assertTrue(all(issubclass(x.category, DeprecationWarning) for x in w))
 
     def testRegistrySlice(self):
         r = markdown.util.Registry()
@@ -377,7 +334,7 @@ class RegistryTests(unittest.TestCase):
         r.register(Item('c'), 'c', 40)
         slc = r[1:]
         self.assertEqual(len(slc), 2)
-        self.assertTrue(isinstance(slc, markdown.util.Registry))
+        self.assertIsInstance(slc, markdown.util.Registry)
         self.assertEqual(list(slc), ['b', 'a'])
 
     def testGetIndexForName(self):
@@ -399,39 +356,6 @@ class RegistryTests(unittest.TestCase):
         self.assertEqual(len(r), 2)
         self.assertEqual(list(r), ['b2', 'a'])
 
-    def testRegistryDeprecatedAdd(self):
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-
-            r = markdown.util.Registry()
-            # Add first item
-            r.add('c', Item('c'), '_begin')
-            self.assertEqual(list(r), ['c'])
-            # Added to beginning
-            r.add('b', Item('b'), '_begin')
-            self.assertEqual(list(r), ['b', 'c'])
-            # Add before first item
-            r.add('a', Item('a'), '<b')
-            self.assertEqual(list(r), ['a', 'b', 'c'])
-            # Add before non-first item
-            r.add('a1', Item('a1'), '<b')
-            self.assertEqual(list(r), ['a', 'a1', 'b', 'c'])
-            # Add after non-last item
-            r.add('b1', Item('b1'), '>b')
-            self.assertEqual(list(r), ['a', 'a1', 'b', 'b1', 'c'])
-            # Add after last item
-            r.add('d', Item('d'), '>c')
-            self.assertEqual(list(r), ['a', 'a1', 'b', 'b1', 'c', 'd'])
-            # Add to end
-            r.add('e', Item('e'), '_end')
-            self.assertEqual(list(r), ['a', 'a1', 'b', 'b1', 'c', 'd', 'e'])
-            with self.assertRaises(ValueError):
-                r.add('f', Item('f'), 'badlocation')
-
-            # Check the warnings
-            self.assertEqual(len(w), 7)
-            self.assertTrue(all(issubclass(x.category, DeprecationWarning) for x in w))
-
 
 class TestErrors(unittest.TestCase):
     """ Test Error Reporting. """
@@ -444,12 +368,6 @@ class TestErrors(unittest.TestCase):
         # Reset warning behavior back to default
         warnings.simplefilter('default')
 
-    def testNonUnicodeSource(self):
-        """ Test falure on non-unicode source text. """
-        if not PY3:
-            source = "foo".encode('utf-16')
-            self.assertRaises(UnicodeDecodeError, markdown.markdown, source)
-
     def testBadOutputFormat(self):
         """ Test failure on bad output_format. """
         self.assertRaises(KeyError, markdown.Markdown, output_format='invalid')
@@ -470,7 +388,7 @@ class TestErrors(unittest.TestCase):
         self.assertRaises(TypeError, markdown.Markdown, extensions=[object])
 
     def testDotNotationExtensionWithBadClass(self):
-        """ Test Extension loading with non-existant class name (`path.to.module:Class`). """
+        """ Test Extension loading with non-existent class name (`path.to.module:Class`). """
         self.assertRaises(
             AttributeError,
             markdown.Markdown,
@@ -498,20 +416,16 @@ class testETreeComments(unittest.TestCase):
 
     def setUp(self):
         # Create comment node
-        self.comment = markdown.util.etree.Comment('foo')
-        if hasattr(markdown.util.etree, 'test_comment'):
-            self.test_comment = markdown.util.etree.test_comment
-        else:
-            self.test_comment = markdown.util.etree.Comment
+        self.comment = etree.Comment('foo')
 
     def testCommentIsComment(self):
         """ Test that an ElementTree Comment passes the `is Comment` test. """
-        self.assertTrue(self.comment.tag is markdown.util.etree.test_comment)
+        self.assertIs(self.comment.tag, etree.Comment)
 
     def testCommentIsBlockLevel(self):
         """ Test that an ElementTree Comment is recognized as BlockLevel. """
         md = markdown.Markdown()
-        self.assertFalse(md.is_block_level(self.comment.tag))
+        self.assertIs(md.is_block_level(self.comment.tag), False)
 
     def testCommentSerialization(self):
         """ Test that an ElementTree Comment serializes properly. """
@@ -537,27 +451,64 @@ class testElementTailTests(unittest.TestCase):
 
     def testBrTailNoNewline(self):
         """ Test that last <br> in tree has a new line tail """
-        root = markdown.util.etree.Element('root')
-        br = markdown.util.etree.SubElement(root, 'br')
+        root = etree.Element('root')
+        br = etree.SubElement(root, 'br')
         self.assertEqual(br.tail, None)
         self.pretty.run(root)
         self.assertEqual(br.tail, "\n")
 
 
+class testElementPreCodeTests(unittest.TestCase):
+    """ Element PreCode Tests """
+    def setUp(self):
+        md = markdown.Markdown()
+        self.pretty = markdown.treeprocessors.PrettifyTreeprocessor(md)
+
+    def prettify(self, xml):
+        root = etree.fromstring(xml)
+        self.pretty.run(root)
+        return etree.tostring(root, encoding="unicode", short_empty_elements=False)
+
+    def testPreCodeEmpty(self):
+        xml = "<pre><code></code></pre>"
+        expected = "<pre><code></code></pre>\n"
+        self.assertEqual(expected, self.prettify(xml))
+
+    def testPreCodeWithChildren(self):
+        xml = "<pre><code> <span /></code></pre>"
+        expected = "<pre><code> <span></span></code></pre>\n"
+        self.assertEqual(expected, self.prettify(xml))
+
+    def testPreCodeWithSpaceOnly(self):
+        xml = "<pre><code> </code></pre>"
+        expected = "<pre><code>\n</code></pre>\n"
+        self.assertEqual(expected, self.prettify(xml))
+
+    def testPreCodeWithText(self):
+        xml = "<pre><code> hello</code></pre>"
+        expected = "<pre><code> hello\n</code></pre>\n"
+        self.assertEqual(expected, self.prettify(xml))
+
+    def testPreCodeWithTrailingSpace(self):
+        xml = "<pre><code> hello </code></pre>"
+        expected = "<pre><code> hello\n</code></pre>\n"
+        self.assertEqual(expected, self.prettify(xml))
+
+
 class testSerializers(unittest.TestCase):
     """ Test the html and xhtml serializers. """
 
     def testHtml(self):
         """ Test HTML serialization. """
-        el = markdown.util.etree.Element('div')
+        el = etree.Element('div')
         el.set('id', 'foo<&">')
-        p = markdown.util.etree.SubElement(el, 'p')
+        p = etree.SubElement(el, 'p')
         p.text = 'foo <&escaped>'
         p.set('hidden', 'hidden')
-        markdown.util.etree.SubElement(el, 'hr')
-        non_element = markdown.util.etree.SubElement(el, None)
+        etree.SubElement(el, 'hr')
+        non_element = etree.SubElement(el, None)
         non_element.text = 'non-element text'
-        script = markdown.util.etree.SubElement(non_element, 'script')
+        script = etree.SubElement(non_element, 'script')
         script.text = '<&"test\nescaping">'
         el.tail = "tail text"
         self.assertEqual(
@@ -572,15 +523,15 @@ class testSerializers(unittest.TestCase):
 
     def testXhtml(self):
         """" Test XHTML serialization. """
-        el = markdown.util.etree.Element('div')
+        el = etree.Element('div')
         el.set('id', 'foo<&">')
-        p = markdown.util.etree.SubElement(el, 'p')
+        p = etree.SubElement(el, 'p')
         p.text = 'foo<&escaped>'
         p.set('hidden', 'hidden')
-        markdown.util.etree.SubElement(el, 'hr')
-        non_element = markdown.util.etree.SubElement(el, None)
+        etree.SubElement(el, 'hr')
+        non_element = etree.SubElement(el, None)
         non_element.text = 'non-element text'
-        script = markdown.util.etree.SubElement(non_element, 'script')
+        script = etree.SubElement(non_element, 'script')
         script.text = '<&"test\nescaping">'
         el.tail = "tail text"
         self.assertEqual(
@@ -595,11 +546,11 @@ class testSerializers(unittest.TestCase):
 
     def testMixedCaseTags(self):
         """" Test preservation of tag case. """
-        el = markdown.util.etree.Element('MixedCase')
+        el = etree.Element('MixedCase')
         el.text = 'not valid '
-        em = markdown.util.etree.SubElement(el, 'EMPHASIS')
+        em = etree.SubElement(el, 'EMPHASIS')
         em.text = 'html'
-        markdown.util.etree.SubElement(el, 'HR')
+        etree.SubElement(el, 'HR')
         self.assertEqual(
             markdown.serializers.to_xhtml_string(el),
             '<MixedCase>not valid <EMPHASIS>html</EMPHASIS><HR /></MixedCase>'
@@ -616,17 +567,17 @@ class testSerializers(unittest.TestCase):
 
     def testQNameTag(self):
         """ Test serialization of QName tag. """
-        div = markdown.util.etree.Element('div')
-        qname = markdown.util.etree.QName('http://www.w3.org/1998/Math/MathML', 'math')
-        math = markdown.util.etree.SubElement(div, qname)
+        div = etree.Element('div')
+        qname = etree.QName('http://www.w3.org/1998/Math/MathML', 'math')
+        math = etree.SubElement(div, qname)
         math.set('display', 'block')
-        sem = markdown.util.etree.SubElement(math, 'semantics')
-        msup = markdown.util.etree.SubElement(sem, 'msup')
-        mi = markdown.util.etree.SubElement(msup, 'mi')
+        sem = etree.SubElement(math, 'semantics')
+        msup = etree.SubElement(sem, 'msup')
+        mi = etree.SubElement(msup, 'mi')
         mi.text = 'x'
-        mn = markdown.util.etree.SubElement(msup, 'mn')
+        mn = etree.SubElement(msup, 'mn')
         mn.text = '2'
-        ann = markdown.util.etree.SubElement(sem, 'annotations')
+        ann = etree.SubElement(sem, 'annotations')
         ann.text = 'x^2'
         self.assertEqual(
             markdown.serializers.to_xhtml_string(div),
@@ -645,8 +596,8 @@ class testSerializers(unittest.TestCase):
 
     def testQNameAttribute(self):
         """ Test serialization of QName attribute. """
-        div = markdown.util.etree.Element('div')
-        div.set(markdown.util.etree.QName('foo'), markdown.util.etree.QName('bar'))
+        div = etree.Element('div')
+        div.set(etree.QName('foo'), etree.QName('bar'))
         self.assertEqual(
             markdown.serializers.to_xhtml_string(div),
             '<div foo="bar"></div>'
@@ -654,14 +605,14 @@ class testSerializers(unittest.TestCase):
 
     def testBadQNameTag(self):
         """ Test serialization of QName with no tag. """
-        qname = markdown.util.etree.QName('http://www.w3.org/1998/Math/MathML')
-        el = markdown.util.etree.Element(qname)
+        qname = etree.QName('http://www.w3.org/1998/Math/MathML')
+        el = etree.Element(qname)
         self.assertRaises(ValueError, markdown.serializers.to_xhtml_string, el)
 
     def testQNameEscaping(self):
         """ Test QName escaping. """
-        qname = markdown.util.etree.QName('<&"test\nescaping">', 'div')
-        el = markdown.util.etree.Element(qname)
+        qname = etree.QName('<&"test\nescaping">', 'div')
+        el = etree.Element(qname)
         self.assertEqual(
             markdown.serializers.to_xhtml_string(el),
             '<div xmlns="&lt;&amp;&quot;test&#10;escaping&quot;&gt;"></div>'
@@ -669,8 +620,8 @@ class testSerializers(unittest.TestCase):
 
     def testQNamePreEscaping(self):
         """ Test QName that is already partially escaped. """
-        qname = markdown.util.etree.QName('&lt;&amp;"test&#10;escaping"&gt;', 'div')
-        el = markdown.util.etree.Element(qname)
+        qname = etree.QName('&lt;&amp;"test&#10;escaping"&gt;', 'div')
+        el = etree.Element(qname)
         self.assertEqual(
             markdown.serializers.to_xhtml_string(el),
             '<div xmlns="&lt;&amp;&quot;test&#10;escaping&quot;&gt;"></div>'
@@ -718,8 +669,8 @@ class testAtomicString(unittest.TestCase):
 
     def testString(self):
         """ Test that a regular string is parsed. """
-        tree = markdown.util.etree.Element('div')
-        p = markdown.util.etree.SubElement(tree, 'p')
+        tree = etree.Element('div')
+        p = etree.SubElement(tree, 'p')
         p.text = 'some *text*'
         new = self.inlineprocessor.run(tree)
         self.assertEqual(
@@ -729,8 +680,8 @@ class testAtomicString(unittest.TestCase):
 
     def testSimpleAtomicString(self):
         """ Test that a simple AtomicString is not parsed. """
-        tree = markdown.util.etree.Element('div')
-        p = markdown.util.etree.SubElement(tree, 'p')
+        tree = etree.Element('div')
+        p = etree.SubElement(tree, 'p')
         p.text = markdown.util.AtomicString('some *text*')
         new = self.inlineprocessor.run(tree)
         self.assertEqual(
@@ -740,14 +691,14 @@ class testAtomicString(unittest.TestCase):
 
     def testNestedAtomicString(self):
         """ Test that a nested AtomicString is not parsed. """
-        tree = markdown.util.etree.Element('div')
-        p = markdown.util.etree.SubElement(tree, 'p')
+        tree = etree.Element('div')
+        p = etree.SubElement(tree, 'p')
         p.text = markdown.util.AtomicString('*some* ')
-        span1 = markdown.util.etree.SubElement(p, 'span')
+        span1 = etree.SubElement(p, 'span')
         span1.text = markdown.util.AtomicString('*more* ')
-        span2 = markdown.util.etree.SubElement(span1, 'span')
+        span2 = etree.SubElement(span1, 'span')
         span2.text = markdown.util.AtomicString('*text* ')
-        span3 = markdown.util.etree.SubElement(span2, 'span')
+        span3 = etree.SubElement(span2, 'span')
         span3.text = markdown.util.AtomicString('*here*')
         span3.tail = markdown.util.AtomicString(' *to*')
         span2.tail = markdown.util.AtomicString(' *test*')
@@ -762,7 +713,7 @@ class testAtomicString(unittest.TestCase):
 
 class TestConfigParsing(unittest.TestCase):
     def assertParses(self, value, result):
-        self.assertTrue(markdown.util.parseBoolValue(value, False) is result)
+        self.assertIs(markdown.util.parseBoolValue(value, False), result)
 
     def testBooleansParsing(self):
         self.assertParses(True, True)
@@ -773,8 +724,8 @@ class TestConfigParsing(unittest.TestCase):
         self.assertParses('none', False)
 
     def testPreserveNone(self):
-        self.assertTrue(markdown.util.parseBoolValue('None', preserve_none=True) is None)
-        self.assertTrue(markdown.util.parseBoolValue(None, preserve_none=True) is None)
+        self.assertIsNone(markdown.util.parseBoolValue('None', preserve_none=True))
+        self.assertIsNone(markdown.util.parseBoolValue(None, preserve_none=True))
 
     def testInvalidBooleansParsing(self):
         self.assertRaises(ValueError, markdown.util.parseBoolValue, 'novalue')
@@ -806,7 +757,7 @@ class TestCliOptionParsing(unittest.TestCase):
 
     def testQuietOption(self):
         options, logging_level = parse_options(['-q'])
-        self.assertTrue(logging_level > CRITICAL)
+        self.assertGreater(logging_level, CRITICAL)
 
     def testVerboseOption(self):
         options, logging_level = parse_options(['-v'])
@@ -865,7 +816,7 @@ class TestCliOptionParsing(unittest.TestCase):
 
     def create_config_file(self, config):
         """ Helper to create temp config files. """
-        if not isinstance(config, markdown.util.string_type):
+        if not isinstance(config, str):
             # convert to string
             config = yaml.dump(config)
         fd, self.tempfile = tempfile.mkstemp('.yml')
@@ -964,7 +915,7 @@ class TestAncestorExclusion(unittest.TestCase):
 
         def handleMatch(self, m, data):
             """ Handle match. """
-            el = markdown.util.etree.Element(self.tag)
+            el = etree.Element(self.tag)
             el.text = m.group(2)
             return el, m.start(0), m.end(0)
 
@@ -1004,42 +955,3 @@ Some +test+ and a [+link+](http://test.com)
 
         self.md.reset()
         self.assertEqual(self.md.convert(test), result)
-
-
-class TestGeneralDeprecations(unittest.TestCase):
-    """Test general deprecations."""
-
-    def test_version_deprecation(self):
-        """Test that version is deprecated."""
-
-        with warnings.catch_warnings(record=True) as w:
-            # Cause all warnings to always be triggered.
-            warnings.simplefilter("always")
-            # Trigger a warning.
-            version = markdown.version
-            # Verify some things
-            self.assertTrue(len(w) == 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
-            self.assertEqual(version, markdown.__version__)
-
-    def test_version_info_deprecation(self):
-        """Test that version info is deprecated."""
-
-        with warnings.catch_warnings(record=True) as w:
-            # Cause all warnings to always be triggered.
-            warnings.simplefilter("always")
-            # Trigger a warning.
-            version_info = markdown.version_info
-            # Verify some things
-            self.assertTrue(len(w) == 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
-            self.assertEqual(version_info, markdown.__version_info__)
-
-    def test_deprecation_wrapper_dir(self):
-        """Tests the `__dir__` attribute of the class as it replaces the module's."""
-
-        dir_attr = dir(markdown)
-        self.assertFalse('version' in dir_attr)
-        self.assertTrue('__version__' in dir_attr)
-        self.assertFalse('version_info' in dir_attr)
-        self.assertTrue('__version_info__' in dir_attr)
